/* ******************************************************************************

	FILE:			Minimal.cpp
	AUTHOR:			Jean-Sbastien Perrier

	DESCRIPTION:	Main file. Contains the entry point of the program.
					Also, it contains the callback function for the configuration
					dialog box. 

	$History::                                                                  $

****************************************************************************** */

// Includes ****************************************************************** //
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "resource.h"

#include "SSMinimalApp.h"
#include "SSUtils.h"

// Defines ******************************************************************* //

// Types ********************************************************************* //

// Consts ******************************************************************** //

// Globals ******************************************************************* //

// Static Members Storage Point ********************************************** //

// Function Prototypes ******************************************************* //

static BOOL CALLBACK ConfigDialogProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam);

// Module External Functions ************************************************* //


/////////////////////////////////////////////////////////////////////////////////
// FUNCTION:	WinMain
// DESCRIPTION:	Program's entry point.
// REMARKS:		None
/////////////////////////////////////////////////////////////////////////////////
int WINAPI WinMain(HINSTANCE a_hInst, HINSTANCE,LPSTR,int a_nCmdShow)
{
	// Create the application instance.
	SSMinimalApp *pApp = new SSMinimalApp(a_hInst);

	// Load application settings (or set defaults value if no
	// registry was previously created).
	pApp->ReadConfigRegistry();

	// Process the command line to know running mode (preview, config, etc...)
	HWND hParentWnd = pApp->ProcessCommandLine(GetCommandLine());

	// Handle the different running mode:
	switch(pApp->GetMode())
	{
	case SSBaseApp::smPassword:
		SSUtl_ChangePassword(hParentWnd);
		break;
	case SSBaseApp::smConfig:
		DialogBox(a_hInst, MAKEINTRESOURCE(DLG_CONFIG), hParentWnd, ConfigDialogProc);
		break;
	case SSBaseApp::smSaver:
	case SSBaseApp::smPreview:	
		pApp->RunScreenSaver(hParentWnd, a_nCmdShow);
		break;
	default:
		MessageBox(NULL, "Something when wrong in parsing cmd line, \nbailing out...", "Error", MB_OK);
		break;
	}

	// Delete the application before quitting.
	delete pApp;
	return 0;
}

// Module Internal Functions ************************************************* //

// Callback function for the dialog box.
BOOL CALLBACK ConfigDialogProc(HWND hwnd,UINT msg, WPARAM wParam, LPARAM lParam)
{
	// Get the application's instance.
	SSMinimalApp *pApp = (SSMinimalApp *)SSBaseApp::GetGlobalInstance();
	switch (msg)
	{
	case WM_INITDIALOG:
		{
			// Get the values from the application and propagate them
			// into the dialog box controls.
			CheckDlgButton(hwnd,IDC_CHECK_ROTATE, pApp->GetRotate());
			CheckDlgButton(hwnd,IDC_CHECK_FLASH, pApp->GetFlash());
			SetDlgItemText(hwnd, IDC_EDIT_TEXTUREFILE, pApp->GetTextureFile());
			return true;
		}
    case WM_COMMAND:
		{
			int id = LOWORD(wParam);
			if( id==IDOK )
			{
				// Get the values from the dialog box controls and
				// propagate them into the application.
				char szBuffer[256];
				pApp->SetRotate(IsDlgButtonChecked(hwnd,IDC_CHECK_ROTATE)==BST_CHECKED);
				pApp->SetFlash(IsDlgButtonChecked(hwnd,IDC_CHECK_FLASH)==BST_CHECKED);
				// Read the file name specified.
				GetDlgItemText(hwnd, IDC_EDIT_TEXTUREFILE, szBuffer, 256);
				pApp->SetTextureFile(szBuffer);

				// Then, save the configuration.
				pApp->WriteConfigRegistry();
			}

			if( id==IDOK || id==IDCANCEL )	EndDialog(hwnd,id);
		} break;
	}
	return false;
}